<?php
// (c) 2005 Adam Peller - peller@gnu.org

// Careful with the sketch state.  If you use draw/command out of 
// context, your connection will hang.  Need to put in checks.

class SoundBridge {

 var $PORT = 4444;
 var $fp;
 var $sketch = false;

 function SoundBridge($ip) {
  $this->fp = fsockopen($ip, $this->PORT, $errno, $errstr, 30);
  if (!$this->fp) {
    echo "Error opening socket to SoundBridge: $errstr ($errno)";
  }
 }

 function enterSketch($clear = true) {
  if ($this->sketch)
   return false;

  $this->draw("sketch");
  $this->sketch = true;
  if ($clear)
   $this->draw("clear");
  return true;
 }

 function exitSketch() {
  if (!$this->sketch)
   return false;
  $this->command("quit");
  return true;
 }

 function close() {
  $this->command("exit");
  fclose($this->fp);
 }

 function command($statement) {
  fwrite($this->fp, $statement."\n");
  $this->waitFor("SoundBridge>");
 }

 function draw($statement) {
  fwrite($this->fp, $statement."\n");
  $this->waitFor("sketch>");
 }

 function clearRect($x, $y, $w, $h) {
  $this->draw("color 0");
  $this->draw("rect $x $y $w $h");
  $this->draw("color 1");
 }

 private function waitFor($prompt) {
  $out = '';

  while (!feof($this->fp) and $out .= fread($this->fp, 1)) {
    if (!(strpos($out, $prompt) === FALSE)) {
      return $out;
    }
  }

  return false;
 }

 function drawMatrix(&$matrix, $threshold, $x0, $y0, $xStep, $yStep) {
  $y1 = $y0;

  for ($ySrc = 0; $ySrc < count($matrix); $ySrc += $yStep) {
   $row = $matrix[$ySrc];
   $x1 = $x0;

   for ($xSrc = 0; $xSrc < count($row);) {
    $run = 0;
    while ((($xSrc + $run * $xStep) < count($row))
          and ($row[$xSrc + $run * $xStep] < $threshold))
     $run++;

    if ($run > 1) {
     $this->draw("line $x1 $y1 ".($x1 + $run)." $y1");
     $x1 += $run;
     $xSrc += $run * $xStep;
     continue;
    }

    if ($run == 1) {
     $this->draw("point $x1 $y1");
    }

    $x1++; $xSrc += $xStep;
   }
   $y1++;
  }
 }

 function wrap($text, $x=0)
 {
  //46 chars/line minus (x offset / 6 pixels per char)
  $text = wordwrap($text, floor(46 - $x/6), "\n", 1);
  $lines = explode("\n", $text);
  $this->draw("text $x 0 \"".$lines[0]."\"");
  if (count($lines) > 1)
   $this->draw("text $x 9 \"".$lines[1]."\"");
 }

 function marquee($text, $delay)
 {
  //116 char limit.  Chunk as <116 marquees, remainder as plain centered text
  $text = wordwrap($text, 113, "\n", 1);
  $lines = explode("\n", $text);
  $num = count($lines);
  foreach ($lines as $line) {
    if (strlen($line) <= 46) {
      $this->draw("text 0 5 \"$line\"");
      sleep($delay);
    } else {
      if (--$num > 0)
        $line .= "...";
      $this->draw("marquee \"$line\"");
    }
  }
 }

}

?>

